import 'package:flutter/foundation.dart';
import 'package:dio/dio.dart';
import 'dart:convert';
import 'dart:async';
import 'dart:io';
import 'utils.dart';
import 'config.dart';
import 'report_system_info.dart';

/// 上报错误模块: 错误信息->错误过滤(随机抽样+重复+忽略)->参数格式化->上报队列发送请求
class ReportError {
  List errorQueue = []; // 记录错误队列
  Map repeatList = {}; // 记录重复异常数据
  Map config;
  ReportSystemInfo reportSystemInfo;

  // 网络请求插件
  Dio dio;

  ReportError({options}) {
    this.config = Config(options: options).config;
    this.dio = Dio(BaseOptions(
      connectTimeout: 10000,
      receiveTimeout: 3000,
    ));
    this.reportSystemInfo = ReportSystemInfo();
  }

  // 动态设置config
  setConfig(Map options) {
    if (options == null) {
      return;
    }
    if (options['key'] == null || options['key'].isEmpty) {
      print('上报配置错误: please set app key!');
      return;
    }
    this.config.addAll(options);
  }

  // 设置单个配置
  setConfigOption({String name, value}) {
    this.config[name] = value;
    return this.config[name];
  }

  // 外部可设置系统 定位+基础版本库信息 locationInfo frameVersion
  setReportSystemOption({Map locationInfo, String frameVersion = '0.0.0'}) {
    this.reportSystemInfo.locationInfo = locationInfo;
    this.reportSystemInfo.frameVersion = frameVersion;
    return this.reportSystemInfo;
  }

  // 构建错误信息
  FlutterErrorDetails makeErrorDetail(Object obj, StackTrace stack) {
    return FlutterErrorDetails(exception: obj, stack: stack);
  }

  // 上传错误信息日志
  reportErrorAndLog(FlutterErrorDetails details) {
    if (details == null) {
      return false;
    }
    //print('上传错误信息日志:${details.toString()}');
    String _stackStr =
        details.stack.toString(); // 堆栈信息 details.stack.toString()

    Map errorMap = {
      'msg': details.exceptionAsString(), // 错误名称
      'stack': _stackStr,
      'type': this.config['errorType'],
      'level': this.config['errorLevel'],
      'page': Utils.getStackPage(_stackStr),
      'ts': Utils.now().millisecondsSinceEpoch, // 13位时间戳
      'api': '', // 接口地址 （接口类错误专用）
      'data': '' // 接口参数 （接口类错误专用）
    };
    // 错误信息过滤拦截
    if (_filterError(errorMap)) {
      // 队列发送请求
      _delayedSend();
    }
    return errorMap;
  }

  // 错误信息过滤
  bool _filterError(error) {
    // 当前用户错误率抽样率控制
    double randomNum = Utils.getRandom();
    if (randomNum >= this.config['random']) {
      return false;
    }

    if (this._repeat(error)) {
      return false;
    }

    if (this._except(error)) {
      return false;
    }

    this.errorQueue.add(error);
    return true;
  }

  // 重复出现的错误，只上报config.repeat次
  bool _repeat(error) {
    String _repeatName = error['msg'].toString();
    this.repeatList[_repeatName] = this.repeatList[_repeatName] != null
        ? this.repeatList[_repeatName] + 1
        : 1;
    return this.repeatList[_repeatName] > this.config['repeat'];
  }

  // 忽略错误
  bool _except(error) {
    List oExcept = this.config['except'];
    bool result = false;
    var v;
    for (var i = 0, len = oExcept.length; i < len; i++) {
      v = oExcept[i];
      if ((v is RegExp) && v.hasMatch(error.msg)) {
        result = true;
        break;
      }
    }
    return result;
  }

  // 延时队列
  void _delayedSend() {
    if (this.errorQueue.length < this.config['mergeSize']) {
      Timer t;
      t = Timer(Duration(milliseconds: this.config['delay']), () {
        _report();
        t.cancel();
      });
    } else {
      _report();
    }
  }

  // 上报错误信息
  void _report() {
    if (this.errorQueue.length == 0) {
      return;
    }
    // 统一参数处理
    // s - 系统各项信息，按顺序为 应用版本号 | 网络 | 基础库版本 | 当前电量 | 手机类型 | 手机完整型号 | 手机操作系统版本 | 微信版本
    String systemInfo =
        '${this.config['version']}${this.reportSystemInfo.appVersion}|'
        '${this.reportSystemInfo.network}|'
        '${this.reportSystemInfo.frameVersion}|'
        '${this.reportSystemInfo.battery}|'
        '${this.reportSystemInfo.phoneType}|'
        '${this.reportSystemInfo.phoneName}|'
        '${this.reportSystemInfo.phoneSystem}|'
        '${this.reportSystemInfo.weChatVersion}';

    var latitude = this.reportSystemInfo.locationInfo['latitude'];
    var longitude = this.reportSystemInfo.locationInfo['longitude'];
    String userId = this.config['userId'];
    String key = this.config['key'];
    String guid = this.config['guid'];
    List errors = [];
    this.errorQueue.forEach((it) {
      var item = [
        it['ts'],
        it['type'],
        it['page'],
        it['msg'],
        it['level'],
        it['stack'],
        it['api'],
        it['data']
      ];
      errors.add(item);
    });
    Map _params = {
      'e': errors,
      'key': key,
      's': systemInfo,
      'l': [latitude, longitude],
      'g': guid,
      'u': userId,
      'extra': this.config['extra'],
    };
    this.errorQueue.clear();
    _request(url: this.config['url'], params: _params);
  }

  // 请求服务端
  void _request(
      {String url,
      Map params,
      Function success,
      Function fail,
      Function complete}) async {
    try {
      print(
          '上报请求参数：${params['key']}、${params['s']}、${params['l']}、${params['g']}、${params['u']}、${params['extra']}');
      print('${params['e']}');
      // 校验唯一参数key
      if (params['key'] == null || params['key'].isEmpty) {
        print('error: please set app key!');
        return;
      }

      print('上报请求完整参数:$url');
      // print(params);

      Response response = await this.dio.post(url,
          data: this.config['reportDecode'] ? jsonEncode(params) : params,
          // 坑点：平台必须转义
          options: Options(
              contentType: ContentType.parse(this.config['contentType'])));
      print('上报请求成功:$response');
    } catch (e) {
      print('上报请求出错了: ${e.toString()}');
    }
  }
}
