import 'dart:io';
import 'package:device_info/device_info.dart';
import 'package:battery/battery.dart';
import 'package:connectivity/connectivity.dart';
import 'package:package_info/package_info.dart';

/// 系统各项信息，按顺序为 应用版本号 | 网络 | 基础库版本 | 当前电量 | 手机类型 | 手机完整型号 | 手机操作系统版本 | 微信版本

class ReportSystemInfo {
  String _appVersion = '';
  String _network = '';
  int _batteryLevel;

  String _phoneType = '';
  String _phoneFullName = '';
  String _phoneSystem = '';
  var _phoneSDK;
  String _frameVersion = '0.0.0';
  Map _locationInfo = {'latitude': 0, 'longitude': 0};

  ReportSystemInfo() {
    this._packageInfoInit();
    this._deviceInfoInit();
    this._networkInit();
    this._batteryInit();
  }

  // 应用版本号
  String get appVersion {
    return this._appVersion;
  }

  // 网络
  String get network {
    return this._network;
  }

  // 外部可设置 基础库版本 // 自动获取flutter SDK 版本库
  String get frameVersion {
    return this._frameVersion;
  }

  set frameVersion(String val) {
    this._frameVersion = val;
  }

  // 当前电量
  String get battery {
    return _batteryLevel.toString();
  }

  // 手机类型(手机品牌设备名称)
  String get phoneType {
    return this._phoneType;
  }

  // 手机完整型号
  String get phoneName {
    return this._phoneFullName;
  }

  // 手机操作系统版本
  String get phoneSystem {
    return this._phoneSystem;
  }

  // 微信版本/SDK版本
  String get weChatVersion {
    return this._phoneSDK.toString();
  }

  // 外部可设置 地理经纬度
  Map get locationInfo {
    return this._locationInfo;
  }

  set locationInfo(val) {
    this._locationInfo = val;
  }

  /// 初始化
  void _packageInfoInit() async {
    try {
      PackageInfo packageInfo = await PackageInfo.fromPlatform();
      this._appVersion = '${packageInfo.appName}_${packageInfo.version}';
    } catch (e) {
      print('_packageInfoInit error:${e.toString()}');
    }
  }

  void _networkInit() async {
    try {
      ConnectivityResult connectivityResult =
          await (Connectivity().checkConnectivity());
      if (connectivityResult == ConnectivityResult.mobile) {
        this._network = 'moblie';
      } else if (connectivityResult == ConnectivityResult.wifi) {
        this._network = 'wifi';
      } else {
        this._network = 'none';
      }
    } catch (e) {
      print('_networkInit error:${e.toString()}');
    }
  }

  void _batteryInit() async {
    try {
      _batteryLevel = await Battery().batteryLevel;
    } catch (e) {
      print('_batteryInit error:${e.toString()}');
    }
  }

  void _deviceInfoInit() async {
    try {
      DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
      if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await deviceInfo.iosInfo;
        this._phoneType = iosInfo.name;
        this._phoneFullName = iosInfo.model;
        this._phoneSystem = '${iosInfo.systemName} ${iosInfo.systemVersion}';
        this._phoneSDK = iosInfo.systemVersion;
      } else if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;
        //  设备名称
        this._phoneType = androidInfo.brand;
        //  型号 _deviceInfo.board / ._deviceInfo.device / _deviceInfo.model(最终产品用户可见名称)
        this._phoneFullName = '${androidInfo.id}';
        //  Android 发布版本
        this._phoneSystem = 'Android ${androidInfo.version.release}';
        this._phoneSDK = androidInfo.version.sdkInt;
      }
      // print('==设备信息:==${this._phoneType}|${this._phoneFullName}|${this._phoneSystem}|${this._phoneSDK}');
    } catch (e) {
      print('_deviceInfoInit error:${e.toString()}');
    }
  }
}
